import datamaestro


class Repository(datamaestro.Repository):
    AUTHOR = """Benjamin Piwowarski <benjamin@piwowarski.fr>"""
    DESCRIPTION = """Text datasets repository"""
    NAMESPACE = """text"""
